@@ -1,8 +1,9 @@
-#define VERSION "1.77"
+#define VERSION "1.81"
 
 /*
  *  units, a program for units conversion
- *  Copyright (C) 1996, 1997, 1999, 2000, 2001 Free Software Foundation, Inc
+ *  Copyright (C) 1996, 1997, 1999, 2000, 2001, 2002 
+ *  Free Software Foundation, Inc
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -22,6 +23,7 @@
  */
 
 #include<stdio.h>
+#include<signal.h>
 
 #ifdef READLINE
 #  include <readline/readline.h>
@@ -42,11 +44,15 @@
 #endif
 
 #define PRIMITIVECHAR '!'	/* Character that marks irreducible units */
+#define COMMENTCHAR '#'         /* Comments marked by this character */
+#define COMMANDCHAR '!'         /* Unit database commands marked with this */
 #define HELPCOMMAND "help"      /* Command to request help at prompt */
 #define UNITMATCH "?"           /* Command to request conformable units */
 #define DEFAULTPAGER "more"     /* Default pager program */
+#define DEFAULTLOCALE "en_US"   /* Default locale */
 
 char *pager;                    /* Actual pager (from PAGER environment var) */
+char *mylocale;                 /* Locale in effect (from LOCALE env var) */
 char *numformat = "%.8g";	/* printf format for numeric output */
 char *powerstring = "^";	/* Exponent character used in output */
 int quiet = 0;                  /* Flag for supressing prompting (-q option) */
@@ -355,7 +361,8 @@ readunits() /* char *userfile) */
    unsigned hashval, pval;
    int unitcount, prefixcount, funccount;
    struct func *funcentry;
-
+   int wronglocale = 0;   /* If set then we are currently reading data */
+                          /* for the wrong locale so we should skip it */
    unitcount = 0;
    prefixcount = 0;
    funccount  = 0;
@@ -414,9 +421,21 @@ readunits() /* char *userfile) */
    while (!feof(unitfile)) {
       if (!fgetslong(&line, &linebufsize, unitfile, &linenum)) 
         break;
-      if (*line == '/')    /* units file comment */
-	 continue;
-      if (lineptr = strchr(line,'#'))
+      if (*line == COMMANDCHAR) {         /* Process units.dat commands    */
+        unitname = strtok(line+1, WHITE);
+	if (!strcmp(unitname,"locale")){  /* The only commands right now   */
+	  unitname = strtok(0, WHITE);    /* provide locale support.       */
+	  if (strcmp(unitname,mylocale))  /* locales don't match           */
+	    wronglocale = 1;
+	} 
+	else if (!strcmp(unitname, "endlocale"))
+	  wronglocale = 0;
+	else readerror(linenum,unitsfilename);
+	continue;
+      }	
+      if (wronglocale)
+	continue;
+      if (lineptr = strchr(line,COMMENTCHAR))
          *lineptr = 0;
       unitname = strtok(line, WHITE);
       if (!unitname || !*unitname) 
@@ -1011,7 +1030,9 @@ reduceunit(struct unittype *theunit)
    /* Keep calling reduceprodut until it doesn't do anything */
 
    while (ret & DIDREDUCTION) {
-      ret = reduceproduct(theunit, 0) | reduceproduct(theunit, 1);
+      ret = reduceproduct(theunit, 0);
+      if (!(ret & ERROR))
+	ret |= reduceproduct(theunit, 1);
       if (ret & ERROR){
          if (irreducible) 
 	   return E_UNKNOWNUNIT;
@@ -1814,6 +1835,9 @@ tryallunits(struct unittype *have)
   outfile = 0;
   if (count==0)
     printf("No matching units found.\n");
+#ifdef SIGPIPE
+  signal(SIGPIPE, SIG_IGN);
+#endif
   if (count>screensize()){
     outfile = popen(pager, "w");
   }
@@ -1827,6 +1851,9 @@ tryallunits(struct unittype *have)
   }
   if (outfile != stdout)
     pclose(outfile);
+#ifdef SIGPIPE
+  signal(SIGPIPE, SIG_DFL);
+#endif
 }
 
 
@@ -2224,6 +2251,10 @@ main(int argc, char **argv)
 
    interactive = processargs(argc, argv, &havestr, &wantstr);
 
+   mylocale = getenv("LOCALE");
+   if (!mylocale)
+     mylocale = DEFAULTLOCALE;
+
    readunits(); /*userfile);*/
 
    if (unitcheck) {
@@ -2333,3 +2364,8 @@ crude example to convert lines in a file 'feet' from ft to m might be:
 
 #endif
 
+/* suggestions:
+    support conversion to a list of units, eg   "ft,in"
+    have units do "spelling advice" if you give a nonconformable conversion
+*/
+
